@@ -835,6 +835,11 @@ void UResetCTM(Matrix *ctm)
    ctm->a = ctm->e = 1;
    ctm->b = ctm->d = 0;
    ctm->c = ctm->f = 0;  /* 0.5 for nearest-int real->int conversion? */
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*------------------------------------------------------------------------*/
@@ -854,6 +859,11 @@ void InvertCTM(Matrix *ctm)
    ctm->e = tmpa / det;
    ctm->c = tx / det;
    ctm->f = ty / det;
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*------------------------------------------------------------------------*/
@@ -867,6 +877,11 @@ void UCopyCTM(fctm, tctm)
    tctm->d = fctm->d;
    tctm->e = fctm->e;
    tctm->f = fctm->f;
+
+#ifdef HAVE_CAIRO
+   if (tctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(tctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*-------------------------------------------------------------------------*/
@@ -884,6 +899,11 @@ void UMakeWCTM(Matrix *ctm)
    ctm->e *= -areawin->vscale;
    ctm->f = (float)areawin->height + ((float)areawin->pcorner.y - ctm->f) *
 	    areawin->vscale;
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*------------------------------------------------------------------------*/
@@ -912,6 +932,11 @@ void UMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
    ctm->a = mata;
    ctm->b = matb;
    ctm->c = matc;
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*----------------------------------------------------------------------*/
@@ -922,6 +947,11 @@ void USlantCTM(Matrix *ctm, float beta)
 {
    ctm->b += ctm->a * beta;
    ctm->e += ctm->d * beta;
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 #define EPS 1e-9
@@ -950,6 +980,10 @@ void UPreScaleCTM(Matrix *ctm)
    /* with the PostScript output due to roundoff error.  This code	*/
    /* matches what PostScript produces.					*/
 
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*----------------------------------------------------------------------*/
@@ -1001,6 +1035,11 @@ void UPreMultCTM(Matrix *ctm, XPoint position, float scale, short rotate)
 
    ctm->a = mata;
    ctm->d = matd;
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*----------------------------------------------------------------------*/
@@ -1020,6 +1059,11 @@ void UPreMultCTMbyMat(Matrix *ctm, Matrix *pre)
    ctm->f += pre->c * ctm->d + pre->f * ctm->e;
    ctm->e = pre->b * ctm->d + pre->e * ctm->e;
    ctm->d = matd;
+
+#ifdef HAVE_CAIRO
+   if (ctm == DCTM && areawin->redraw_ongoing)
+      xc_cairo_set_matrix(ctm);
+#endif /* HAVE_CAIRO */
 }
 
 /*------------------------------------------------------------------------*/
@@ -1071,6 +1115,10 @@ void UPopCTM()
    lastmatrix = areawin->MatStack->nextmatrix;
    free(areawin->MatStack);
    areawin->MatStack = lastmatrix;
+
+#ifdef HAVE_CAIRO
+   xc_cairo_set_matrix(lastmatrix);
+#endif /* HAVE_CAIRO */
 }
 
 /*------------------------------------------------------------------------*/
@@ -1315,8 +1363,8 @@ void labelbbox(labelptr labox, XPoint *npoints, objinstptr callinst)
 void graphicbbox(graphicptr gp, XPoint *npoints)
 {
    XPoint points[4];
-   int hw = gp->source->width >> 1;
-   int hh = gp->source->height >> 1;
+   int hw = xcImageGetWidth(gp->source) >> 1;
+   int hh = xcImageGetHeight(gp->source) >> 1;
 
    points[1].x = points[2].x = hw;
    points[0].x = points[3].x = -hw;
@@ -1808,6 +1856,7 @@ void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 /* Basic X Graphics Routines in the User coordinate system */
 /*---------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
 {
    XPoint newpt1, newpt2;
@@ -1818,9 +1867,11 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
    DrawLine(dpy, areawin->window, areawin->gc,
 		newpt1.x, newpt1.y, newpt2.x, newpt2.y); 
 } 
+#endif /* !HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawLine(XPoint *pt1, XPoint *pt2)
 {
    float tmpwidth = UTopTransScale(xobjs.pagelist[areawin->page]->wirewidth);
@@ -1828,6 +1879,7 @@ void UDrawLine(XPoint *pt1, XPoint *pt2)
    SetLineAttributes(dpy, areawin->gc, tmpwidth, LineSolid, CapRound, JoinBevel);
    UDrawSimpleLine(pt1, pt2); 
 } 
+#endif /* !HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Add circle at given point to indicate that the point is a parameter.	*/
@@ -1843,6 +1895,7 @@ void UDrawLine(XPoint *pt1, XPoint *pt2)
 /* negative = clockwise)).						*/
 /*----------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawCircle(XPoint *upt, u_char which)
 {
    XPoint wpt;
@@ -1869,11 +1922,13 @@ void UDrawCircle(XPoint *upt, u_char which)
 	 break;
    }
 }
+#endif /* !HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Add "X" at string origin						*/
 /*----------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawXAt(XPoint *wpt)
 {
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapButt, JoinMiter);
@@ -1882,6 +1937,7 @@ void UDrawXAt(XPoint *wpt)
    DrawLine(dpy, areawin->window, areawin->gc, wpt->x + 3,
 		wpt->y - 3, wpt->x - 3, wpt->y + 3);
 }
+#endif /* !HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Draw "X" on current level						*/
@@ -2016,8 +2072,12 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    UPreMultCTM(DCTM, curlabel->position, curlabel->scale, curlabel->rotation);
    tmpjust = flipadjust(curlabel->justify);
 
+#ifdef HAVE_CAIRO
+   SetForeground(dpy, areawin->gc, AUXCOLOR);
+#else /* HAVE_CAIRO */
    SetFunction(dpy, areawin->gc, GXxor);
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+#endif /* HAVE_CAIRO */
 
    tmpext = ULength(curlabel, areawin->topinstance, tpos, NULL);
    xdist = tmpext.width;
@@ -2062,6 +2122,7 @@ void UDrawTLine(labelptr curlabel)
 /* Draw an X		*/
 /*----------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawXLine(XPoint opt, XPoint cpt)
 {
    XPoint upt, vpt;
@@ -2084,9 +2145,11 @@ void UDrawXLine(XPoint opt, XPoint cpt)
    SetFunction(dpy, areawin->gc, areawin->gctype);
    SetForeground(dpy, areawin->gc, areawin->gccolor);
 }
+#endif /* HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawBox(XPoint origin, XPoint corner)
 {
    XPoint	worig, wcorn;
@@ -2097,7 +2160,6 @@ void UDrawBox(XPoint origin, XPoint corner)
    SetFunction(dpy, areawin->gc, GXxor);
    SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
    SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
-
    DrawLine(dpy, areawin->window, areawin->gc, worig.x, worig.y,
 		worig.x, wcorn.y);
    DrawLine(dpy, areawin->window, areawin->gc, worig.x, wcorn.y,
@@ -2107,15 +2169,15 @@ void UDrawBox(XPoint origin, XPoint corner)
    DrawLine(dpy, areawin->window, areawin->gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
+#endif /* HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
-/* Draw a box indicating the dimensions of the edit element that most	*/
+/* Get a box indicating the dimensions of the edit element that most	*/
 /* closely reach the position "corner".					*/
 /*----------------------------------------------------------------------*/
 
-float UDrawRescaleBox(XPoint *corner)
+float UGetRescaleBox(XPoint *corner, XPoint *newpoints)
 {
-   XPoint 	origpoints[5], newpoints[5];
    genericptr	rgen;
    float	savescale, newscale;
    long		mindist, testdist, refdist;
@@ -2126,10 +2188,6 @@ float UDrawRescaleBox(XPoint *corner)
 
    if (areawin->selects == 0) return 0.0;
 
-   SetFunction(dpy, areawin->gc, GXxor);
-   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
-   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
-   
    /* Use only the 1st selection as a reference to set the scale */
 
    rgen = SELTOGENERIC(areawin->selectlist);
@@ -2232,12 +2290,37 @@ float UDrawRescaleBox(XPoint *corner)
          break;
    }
 
+   return newscale;
+}
+
+/*----------------------------------------------------------------------*/
+/* Draw a box indicating the dimensions of the edit element that most	*/
+/* closely reach the position "corner".					*/
+/*----------------------------------------------------------------------*/
+
+#ifndef HAVE_CAIRO
+float UDrawRescaleBox(XPoint *corner)
+{
+   XPoint origpoints[5], newpoints[5];
+   float newscale;
+
+   if (areawin->selects == 0) return 0.0;
+
+   newscale = UGetRescaleBox(corner, newpoints);
+
+   SetFunction(dpy, areawin->gc, GXxor);
+   SetForeground(dpy, areawin->gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areawin->gc, 0, LineSolid, CapRound, JoinBevel);
+   
    UTransformbyCTM(DCTM, newpoints, origpoints, 4);
    strokepath(origpoints, 4, 0, 1);
    return newscale;
 }
+#endif /* HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
+
+#ifndef HAVE_CAIRO
 void UDrawBBox()
 {
    XPoint	origin;
@@ -2266,11 +2349,13 @@ void UDrawBBox()
    DrawLine(dpy, areawin->window, areawin->gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
+#endif /* !HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Fill and/or draw a border around the stroking path			*/
 /*----------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void strokepath(XPoint *pathlist, short number, short style, float width)
 {
    float        tmpwidth;
@@ -2365,6 +2450,7 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
       }
    }
 }
+#endif /* !HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
 
@@ -2379,6 +2465,7 @@ void makesplinepath(splineptr thespline, XPoint *pathlist)
 
 /*-------------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawSpline(splineptr thespline, float passwidth)
 {
    XPoint       tmppoints[SPLINESEGS];
@@ -2393,9 +2480,11 @@ void UDrawSpline(splineptr thespline, float passwidth)
       UDrawXLine(thespline->ctrl[3], thespline->ctrl[2]);
    }
 }
+#endif /* HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawPolygon(polyptr thepoly, float passwidth)
 {
    XPoint *tmppoints = (pointlist) malloc(thepoly->number * sizeof(XPoint));
@@ -2407,9 +2496,11 @@ void UDrawPolygon(polyptr thepoly, float passwidth)
    strokepath(tmppoints, thepoly->number, thepoly->style, scaledwidth);
    free(tmppoints);
 }
+#endif /* HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawArc(arcptr thearc, float passwidth)
 {
    XPoint  tmppoints[RSTEPS + 2];
@@ -2423,9 +2514,11 @@ void UDrawArc(arcptr thearc, float passwidth)
       UDrawXLine(thearc->position, areawin->save);
    }
 }
+#endif /* HAVE_CAIRO */
 
 /*-------------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawPath(pathptr thepath, float passwidth)
 {
    XPoint	*tmppoints = (pointlist) malloc(sizeof(XPoint));
@@ -2481,6 +2574,7 @@ void UDrawPath(pathptr thepath, float passwidth)
    strokepath(tmppoints, pathsegs, thepath->style, scaledwidth);
    free(tmppoints);
 }
+#endif /* HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Main recursive object instance drawing routine.			*/
@@ -2492,6 +2586,7 @@ void UDrawPath(pathptr thepath, float passwidth)
 /*    stack contains graphics context information			*/
 /*----------------------------------------------------------------------*/
 
+#ifndef HAVE_CAIRO
 void UDrawObject(objinstptr theinstance, short level, int passcolor,
 		float passwidth, pushlistptr *stack)
 {
@@ -2692,6 +2787,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor,
       pop_stack(stack);
    }
 }
+#endif /* HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Recursively run through the current page and find any labels which	*/
@@ -2755,7 +2851,7 @@ void UDoLatex(objinstptr theinstance, short level, FILE *f,
 		       xfpos.y += 0.056;
 		       xfpos.x /= scale2;
 		       xfpos.y /= scale2;
-		       ltext = textprint(thislabel->string, theinstance);
+		       ltext = textprinttex(thislabel->string, theinstance);
 		       tbjust = thislabel->justify & (NOTBOTTOM | TOP);
 		       lrjust = thislabel->justify & (NOTLEFT | RIGHT);
 
@@ -2834,7 +2930,7 @@ void TopDoLatex()
 
    /* Modified to use \scalebox and \parbox by Alex Tercete, June 2008 */
 
-   fprintf(f, "\\begin{center}\n");
+   // fprintf(f, "\\begin{center}\n");
 
    outscale = xobjs.pagelist[areawin->page]->outscale;
    psscale = getpsscale(outscale, areawin->page);
@@ -2867,8 +2963,8 @@ void TopDoLatex()
    fprintf(f, "   \\scalebox{%g}{\n", outscale);
    fprintf(f, "   \\normalsize\n");
    fprintf(f, "   \\parbox{%gin}{\n", (((float)width * psscale) / 72.0) / outscale);
-   fprintf(f, "   \\includegraphics[scale=%g]{%s%s}\\\\\n", 1.0 / outscale,
-			filename, extend);
+   fprintf(f, "   \\includegraphics[scale=%g]{%s}\\\\\n", 1.0 / outscale,
+    			filename);
    fprintf(f, "   %% translate x=%d y=%d scale %3.2f\n", tx, ty, psscale);
 
    UPushCTM();		/* Save current state */
@@ -2880,7 +2976,7 @@ void TopDoLatex()
    fprintf(f, "   } %% close \'scalebox\'\n");
    fprintf(f, "   \\vspace{-\\baselineskip} %% this is not"
 		" necessary, but looks better\n");
-   fprintf(f, "\\end{center}\n");
+   // fprintf(f, "\\end{center}\n");
    fclose(f);
 
    Wprintf("Wrote auxiliary file %s.tex", filename);
