@@ -1009,8 +1009,8 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 	          continue;
 	       }
 
-	       XSetFunction(dpy, areawin->gc, GXcopy);
-	       XSetForeground(dpy, areawin->gc, BACKGROUND);
+	       SetFunction(dpy, areawin->gc, GXcopy);
+	       SetForeground(dpy, areawin->gc, BACKGROUND);
 	       easydraw(*sstyle, DOFORALL);
 
 	       *estyle = newstyle;
@@ -1018,8 +1018,12 @@ int setelementstyle(xcWidget w, u_short value, u_short mask)
 	          (SELTOPOLY(sstyle))->color = (value & BBOX) ? BBOXCOLOR
 				: DEFAULTCOLOR;
 
-	       XSetFunction(dpy, areawin->gc, GXxor);
-	       XSetForeground(dpy, areawin->gc, SELECTCOLOR ^ BACKGROUND);
+#ifdef HAVE_CAIRO	       
+	       SetForeground(dpy, areawin->gc, SELECTCOLOR);
+#else /* HAVE_CAIRO */
+	       SetFunction(dpy, areawin->gc, GXxor);
+	       SetForeground(dpy, areawin->gc, SELECTCOLOR ^ BACKGROUND);
+#endif /* HAVE_CAIRO */
 	       easydraw(*sstyle, DOFORALL);
 
 #ifdef TCL_WRAPPER
