@@ -293,6 +293,7 @@ extern Boolean loadlibrary(short);
 extern void startloadfile(int);
 extern void normalloadfile(void);
 extern void importfile(void);
+extern void importgraphic(void);
 extern Boolean loadfile(short, int);
 extern void readlabel(objectptr, char *, stringpart **);
 extern void readparams(objectptr, objinstptr, objectptr, char *);
@@ -443,6 +444,7 @@ extern void UDrawTextLine(labelptr, short);
 extern void UDrawTLine(labelptr);
 extern void UDrawXLine(XPoint, XPoint);
 extern void UDrawBox(XPoint, XPoint);
+extern float UGetRescaleBox(XPoint *corner, XPoint *newpoints);
 extern float UDrawRescaleBox(XPoint *);
 extern void UDrawBBox(void);
 extern void strokepath(XPoint *, short, short, float);
@@ -819,6 +821,13 @@ extern graphicptr gradient_field(objinstptr, int, int, int, int);
 extern void invalidate_graphics(objectptr);
 extern void freegraphic(graphicptr);
 extern short *collect_graphics(short *);
+xcImage *xcImageCreate(int width, int height);
+void xcImageDestroy(xcImage *img);
+int xcImageGetWidth(xcImage *img);
+int xcImageGetHeight(xcImage *img);
+void xcImagePutPixel(xcImage *img, int x, int y, u_char r, u_char g, u_char b);
+void xcImageGetPixel(xcImage *img, int x, int y, u_char *r, u_char *g,
+      u_char *b);
 
 /* from flate.c */
 
@@ -938,8 +947,10 @@ extern int findcurfont(int, stringpart *, objinstptr);
 extern stringpart *findtextinstring(char *, int *, stringpart *, objinstptr);
 extern stringpart *findstringpart(int, int *, stringpart *, objinstptr);
 extern void charprint(char *, stringpart *, int);
+extern void charprinttex(char *, stringpart *, int);
 extern char *stringprint(stringpart *, objinstptr);
 extern char *textprint(stringpart *, objinstptr);
+extern char *textprinttex(stringpart *, objinstptr);
 extern char *textprintsubnet(stringpart *, objinstptr, int);
 extern char *textprintnet(char *, char *, Genericlist *);
 extern int textcomp(stringpart *, char *, objinstptr);
@@ -961,7 +972,7 @@ extern Boolean pin_is_bus(labelptr, objinstptr);
 extern int find_cardinal(int, labelptr, objinstptr);
 extern int find_ordinal(int, labelptr, objinstptr);
 
-extern short UDrawChar(u_char, short, short, int, int, float);
+extern float UDrawChar(u_char, short, short, int, int, float);
 extern void UDrawString(labelptr, int, objinstptr);
 extern void UDrawStringNoX(labelptr, int, objinstptr);
 extern void CheckMarginStop(labelptr, objinstptr, Boolean);
@@ -990,6 +1001,8 @@ extern int rgb_alloccolor(int, int, int);
 extern void addtocolorlist(xcWidget, int);
 extern int addnewcolorentry(int);
 extern int xc_getlayoutcolor(int);
+void xc_get_color_rgb(unsigned long cidx, unsigned short *red, 
+      unsigned short *green, unsigned short *blue);
 extern int query_named_color(char *);
 extern caddr_t CvtStringToPixel(XrmValuePtr, int *, XrmValuePtr, XrmValuePtr);
 extern void outputpopup(xcWidget, caddr_t, caddr_t);
@@ -1015,3 +1028,11 @@ extern void delwin(xcWidget, popupstruct *, XClientMessageEvent *);
 
 
 extern void makecursors(void);
+
+/* from cairo.c */
+
+void xc_cairo_set_matrix(const Matrix *xcm);
+void xc_cairo_set_color(unsigned long coloridx);
+float xc_cairo_get_char_extents(const fontinfo *font, unsigned char c,
+      float *top, float *bottom);
+
