@@ -65,7 +65,7 @@ extern colorindex *colorlist;
 extern Cursor appcursors[NUM_CURSORS];
 
 Pixmap bbuf = (Pixmap)NULL;	/* background buffer */
-#ifdef DOUBLEBUFFER
+#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
 extern Pixmap dbuf;
 #endif
 
@@ -151,6 +151,13 @@ Boolean render_client(XEvent *eventPtr)
       gs_state = GS_INIT;
    } 
    else {
+      char *atomname;
+
+      atomname = XGetAtomName(dpy, eventPtr->xclient.message_type);
+      if (atomname != NULL) {
+	 fprintf(stderr, "Received client message using atom \"%s\"\n", atomname);
+	 XFree(atomname);
+      }
       return False;
    }
    return True;
@@ -240,6 +247,7 @@ void ask_for_next()
 
 void start_gs()
 {
+#ifndef HAVE_CAIRO
    int std_out[2], std_err[2], ret;
 #ifdef XC_WIN32
    unsigned int pipe_size = 8196;
@@ -262,7 +270,7 @@ void start_gs()
 
 #else	/* !TCL_WRAPPER */
    if (bbuf != (Pixmap)NULL) XFreePixmap(dpy, bbuf);
-#ifdef DOUBLEBUFFER
+#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
    bbuf = XCreatePixmap(dpy, dbuf, areawin->width, areawin->height,
 		DefaultDepthOfScreen(xcScreen(areawin->area))); 
 #else
@@ -397,6 +405,7 @@ void start_gs()
       }
    }
 #endif
+#endif /* HAVE_CAIRO */
 }
 
 /*--------------------------------------------------------*/
@@ -701,7 +710,7 @@ int copybackground()
    if (is_page(topobject) == -1)
       return -1;
 
-#ifdef DOUBLEBUFFER
+#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
    XCopyArea(dpy, bbuf, dbuf, areawin->gc, 0, 0,
              areawin->width, areawin->height, 0, 0);
 #else
