@@ -41,8 +41,11 @@ extern int *appcolors;
 extern colorindex *colorlist;
 extern char _STR[150];
 
+#ifdef HAVE_CAIRO
+extern const char *utf8encodings[][256];
+#endif
 /* Global value of distance between characters in the font catalog */
-short del;
+short del = 64;
 
 #ifndef TCL_WRAPPER
 
@@ -96,7 +99,7 @@ void joinlabels()
    }
 
    XcSetFunction(GXcopy);
-   XSetForeground(dpy, areawin->gc, BACKGROUND);
+   SetForeground(dpy, areawin->gc, BACKGROUND);
 
    for (jl = areawin->selectlist; jl < areawin->selectlist +
 		areawin->selects; jl++) {
@@ -121,7 +124,7 @@ void joinlabels()
       }
    }
 
-   XSetForeground(dpy, areawin->gc, dest->color);
+   SetForeground(dpy, areawin->gc, dest->color);
    UDrawString(dest, dest->color, areawin->topinstance);
 
    incr_changes(topobject);
@@ -1409,12 +1412,13 @@ stringpart *stringcopyback(stringpart *string, objinstptr thisinst)
 /* draw a single character at 0, 0 using current transformation matrix */
 /*---------------------------------------------------------------------*/
 
-short UDrawChar(u_char code, short styles, short ffont, int groupheight,
+#ifndef HAVE_CAIRO
+float UDrawChar(u_char code, short styles, short ffont, int groupheight,
 	int passcolor, float passwidth)
 {
    objectptr drawchar;
    XPoint alphapts[2];
-   short  localwidth;
+   float localwidth;
    objinst charinst;
 
    if ((ffont >= fontcount) || (fonts[ffont].encoding == NULL))
@@ -1456,6 +1460,7 @@ short UDrawChar(u_char code, short styles, short ffont, int groupheight,
    }
    return localwidth;
 }
+#endif /* HAVE_CAIRO */
 
 /*----------------------------------------------------------------------*/
 /* Draw an entire string, including parameter substitutions		*/
@@ -1467,12 +1472,15 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 {
    stringpart *strptr;
    char *textptr;
-   short  fstyle, ffont, tmpjust, baseline;
+   short  fstyle, ffont, tmpjust;
+   float  baseline;
    int    pos, group = 0;
    int    defaultcolor, curcolor, scolor;
-   short  oldx, oldfont, oldstyle;
+   float  oldx;
+   short  oldfont, oldstyle;
    float  tmpscale = 1.0, natscale = 1.0, tmpthick = 2.0;
-   XPoint newpoint, bboxin[2], bboxout[2];
+   XfPoint newpoint;
+   XPoint  bboxin[2], bboxout[2];
    u_char xm, ym;
    TextExtents tmpext;
    short *tabstops = NULL;
@@ -1521,7 +1529,10 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
    /* them from the circuit point to which they attach.  */
 
    if (drawlabel->pin) {
-      pinadjust(tmpjust, &(newpoint.x), &(newpoint.y), 1);
+      XPoint p = {.x = newpoint.x, .y = newpoint.y};
+      pinadjust(tmpjust, &p.x, &p.y, 1);
+      newpoint.x = p.x;
+      newpoint.y = p.y;
    }
 
    oldx = newpoint.x;
@@ -1668,9 +1679,10 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 		break;
 
 	     case HALFSPACE: case QTRSPACE: {
+		XPoint p = {.x = newpoint.x, .y = newpoint.y};
 		short addx;
 		UPushCTM();
-		UPreMultCTM(DCTM, newpoint, tmpscale, 0);
+		UPreMultCTM(DCTM, p, tmpscale, 0);
 		addx = UDrawChar((u_char)32, fstyle, ffont, group,
 			curcolor, tmpthick);
 		newpoint.x += addx >> ((strptr->type == HALFSPACE) ? 1 : 2);
@@ -1694,9 +1706,10 @@ void UDrawString0(labelptr drawlabel, int passcolor, objinstptr localinst,
 		}
 
 		for (; textptr && *textptr != '\0'; textptr++) {
+		   XPoint p = {.x = newpoint.x, .y = newpoint.y};
 		   pos++;
 		   UPushCTM();
-		   UPreMultCTM(DCTM, newpoint, tmpscale, 0);
+		   UPreMultCTM(DCTM, p, tmpscale, 0);
 
 	           /* Special case of selection:  only substring is	*/
 		   /* drawn in the selection color.			*/
@@ -1773,6 +1786,9 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
    short tabno, numtabs = 0;
    int marginstop = 0;
    Boolean dobreak = FALSE;
+#ifdef HAVE_CAIRO
+   fontinfo *font = NULL;
+#endif /* HAVE_CAIRO */
 
    retext.ascent = retext.descent = retext.base = 0;
    retext.width = retext.maxwidth = 0;
@@ -1811,17 +1827,29 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    xtotal = 0.5;
 	    break;
 	 case HALFSPACE: 
+#ifdef HAVE_CAIRO
+	    if (font && font->font_face)
+	       xtotal += xc_cairo_get_char_extents(font, ' ', NULL, NULL)
+		     * locscale * natscale / 2.;
+	    else
+#endif /* HAVE_CAIRO */
 	    if (somebet) {
 	       chptr = (*(somebet + 32));
 	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
-			* locscale * natscale / 2;
+			* locscale * natscale / 2.;
 	    }
 	    break;
 	 case QTRSPACE:
+#ifdef HAVE_CAIRO
+	    if (font && fonts->font_face)
+	       xtotal += xc_cairo_get_char_extents(font, ' ', NULL, NULL)
+		     * locscale * natscale / 4.;
+	    else
+#endif /* HAVE_CAIRO */
 	    if (somebet) {
 	       chptr = (*(somebet + 32));
 	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x) 
-			* locscale * natscale / 4;
+			* locscale * natscale / 4.;
 	    }
 	    break;
 	 case TABBACKWARD:	/* find first tab value with x < xtotal */
@@ -1848,8 +1876,8 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    break;
 	 case FONT_SCALE:
 	    strscale = natscale * strptr->data.scale;
-	    if (ykern == 0.0)
-	       natscale = strscale;
+	    // if (ykern == 0.0)
+	    //   natscale = strscale;
 	    break;
 	 case KERN:
 	    xtotal += strptr->data.kern[0];
@@ -1861,6 +1889,9 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	       locscale = fonts[strptr->data.font].scale;
 	       if (ykern == 0.0)
 	          natscale = locscale;
+#ifdef HAVE_CAIRO
+	       font = fonts + strptr->data.font;
+#endif /* HAVE_CAIRO */
 	    }
 	    break;
 	 case TEXT_STRING:
@@ -1882,17 +1913,27 @@ TextExtents ULength(labelptr drawlabel, objinstptr localinst,
 	    if (somebet == NULL) break;
 
 	    for (; textptr && *textptr != '\0'; textptr++) {
+	       float advance, top, bottom;
                if (dostop && (locpos >= dostop)) break;
 	       locpos++;
 
-	       chptr = (*(somebet + *textptr));
-	       xtotal += (float)(chptr->bbox.width + chptr->bbox.lowerleft.x)
-			* locscale * strscale;
-	       retext.ascent = max(retext.ascent, (short)(retext.base + ykern +
-			(float)((chptr->bbox.height + chptr->bbox.lowerleft.y)
-			* locscale * strscale)));
-	       retext.descent = min(retext.descent, (short)(retext.base + ykern +
-			(float)(chptr->bbox.lowerleft.y * locscale * strscale)));
+#ifdef HAVE_CAIRO
+	       if (font->font_face)
+		  advance = xc_cairo_get_char_extents(font, *textptr, &top,
+			&bottom);
+               else
+#endif /* HAVE_CAIRO */
+	       {
+	          chptr = *(somebet + *textptr);
+	          advance = chptr->bbox.width + chptr->bbox.lowerleft.x;
+	          top = chptr->bbox.height + chptr->bbox.lowerleft.y;
+	          bottom = chptr->bbox.lowerleft.y;
+               }
+	       xtotal += advance * locscale * strscale;
+	       retext.ascent = max(retext.ascent, (short)(retext.base + ykern
+		     + top * locscale * strscale));
+	       retext.descent = min(retext.descent, (short)(retext.base + ykern
+		     + bottom * locscale * strscale));
 
                if (tbreak != NULL) 
 	          if ((xtotal > tbreak->x) && (retext.base <= tbreak->y)) {
@@ -2172,7 +2213,7 @@ void composefontlib(short cfont)
    objinstptr *drawinst;
    objectptr *curlib, libobj, nullobj;
    objectptr directory = xobjs.libtop[FONTLIB]->thisobject;
-   short visobjects, i, qdel;
+   short visobjects, i;
    polyptr *drawbox;
    pointlist pointptr;
 
@@ -2200,20 +2241,25 @@ void composefontlib(short cfont)
 		* sizeof(genericptr));
    directory->parts = 0;
 
-   /* 0.5 is the default vscale;  16 is no. characters per line */
-   del = min(areawin->width, areawin->height) / (0.5 * 16);
-   qdel = del >> 2;
-
    for (i = 0; i < 256; i++) {
+      stringpart *sfont, *schar;
+      char character[] = {i, 0};
 
-      if ((libobj = fonts[cfont].encoding[i]) == nullobj) continue;
-      
-      NEW_OBJINST(drawinst, directory);
-      instancedefaults(*drawinst, libobj,
-		(i % 16) * del + qdel,		/* X position */
-		-(i / 16) * del + qdel);	/* Y position */
-      drawinst = (objinstptr *)directory->plist + directory->parts - 1;
-      (*drawinst)->color = DEFAULTCOLOR;
+      if (fonts[cfont].encoding[i] == nullobj)
+	 continue;
+     
+      sfont = (stringpart *) malloc(sizeof(stringpart));
+      schar = (stringpart *) malloc(sizeof(stringpart));
+      sfont->type = FONT_NAME;
+      sfont->data.font = cfont;
+      sfont->nextpart = schar;
+      schar->type = TEXT_STRING;
+      schar->data.string = strdup(character);
+      schar->nextpart = NULL;
+      labelptr label = new_label(xobjs.libtop[FONTLIB], sfont, NORMAL,
+		      (i % 16) * del + del / 2, -(i / 16) * del - del / 2 - 32 /2);
+      label->justify = NOTLEFT;
+      label->color = DEFAULTCOLOR;
    }
 
    /* separate characters with gridlines (17 vert., 17 horiz.) */
@@ -2274,7 +2320,7 @@ void fontcat_op(int op, int x, int y)
 
       window_to_user(x, y, &areawin->save);
  
-      chy = -areawin->save.y / del + 1;
+      chy = -areawin->save.y / del;
       chx = areawin->save.x / del;
 
       chx = min(15, chx);
