@@ -129,7 +129,7 @@ extern Tcl_Interp *xcinterp;
 extern XtAppContext app;
 #endif
 
-#ifdef DOUBLEBUFFER
+#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
 extern Pixmap dbuf;
 #endif
 
@@ -710,6 +710,25 @@ int xc_getlayoutcolor(int cidx)
    return locidx;
 }
 
+/*-------------------------------------------------------------------------*/
+/* Obtain red, green and blue values from a color index                    */
+/* the colour values range from 0 to 65535                                 */
+/*-------------------------------------------------------------------------*/
+
+void xc_get_color_rgb(unsigned long cidx, unsigned short *red, 
+      unsigned short *green, unsigned short *blue)
+{
+   XColor loccolor = {
+         .pixel = cidx,
+         .flags = DoRed | DoGreen | DoBlue
+   };
+   XQueryColors(dpy, cmap, &loccolor, 1);
+   *red = loccolor.red;
+   *green = loccolor.green;
+   *blue = loccolor.blue;
+}
+
+
 /*-------------------------------------------------------------------------*/
 /* Hack on the resource database Color allocation			   */
 /*									   */
@@ -1075,10 +1094,12 @@ XCWindowData *create_new_window()
    newwindow->gccolor = 0;
    newwindow->gctype = 0;
    newwindow->time_id = 0;
+#ifndef HAVE_CAIRO
    newwindow->clipmask = (Pixmap)NULL;
    newwindow->pbuf = (Pixmap)NULL;
    newwindow->cmgc = (GC)NULL;
    newwindow->clipped = 0;
+#endif /* !HAVE_CAIRO */
    newwindow->vscale = 1;
    newwindow->pcorner.x = newwindow->pcorner.y = 0;
    newwindow->topinstance = (objinstptr)NULL;
@@ -1450,7 +1471,7 @@ void post_initialize()
    /* set in initmem() should correctly position the empty page in the	 */
    /* middle of the viewing window.					 */
 
-#ifdef DOUBLEBUFFER
+#if !defined(HAVE_CAIRO) && defined(DOUBLEBUFFER)
    if (dbuf == (Pixmap)NULL)
       dbuf = XCreatePixmap(dpy, areawin->window, areawin->width,
 		areawin->height, DefaultDepthOfScreen(xcScreen(areawin->area)));
